/***************************************************** vim:set ts=4 sw=4 sts=4:
  kttsdconfig.cpp
  KControl module for KTTSD configuration
  -------------------
  Copyright : (C) 2002-2003 by José Pablo Ezequiel "Pupeno" Fernández
  -------------------
  Original author: José Pablo Ezequiel "Pupeno" Fernández <pupeno@kde.org>
  Current Maintainer: José Pablo Ezequiel "Pupeno" Fernández <pupeno@kde.org>
 ******************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License.               *
 *                                                                         *
 ***************************************************************************/

// $Id: kcmkttsd.cpp,v 1.10 2004/03/18 05:45:37 cramblitt Exp $

#include <dcopclient.h>

#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qlayout.h>

#include <kcombobox.h>
#include <kparts/componentfactory.h>
#include <kmessagebox.h>
#include <klineedit.h>
#include <kurlrequester.h>
#include <kiconloader.h>
#include <klineeditdlg.h>
#include <kapplication.h>
#include <kprocess.h>
#include <kgenericfactory.h>
#include <ksimpleconfig.h>
#include <kstandarddirs.h>
 
#include <pluginconf.h>
 
#include "kcmkttsd.h"
#include "kcmkttsd.moc"
#include "kcmkttsdwidget.h"

// Some constants
const bool textPreMsgCheckValue = true;
const QString textPreMsgValue = i18n("Text interrumped. Message.");

const bool textPreSndCheckValue = false;
const QString textPreSndValue = "";

const bool textPostMsgCheckValue = true;
const QString textPostMsgValue = i18n("Resuming text.");

const bool textPostSndCheckValue = false;
const QString textPostSndValue = "";

const bool parPreMsgCheckValue = true;
const QString parPreMsgValue = i18n("Paragraph interrumped. Message.");

const bool parPreSndCheckValue = false;
const QString parPreSndValue = "";

const bool parPostMsgCheckValue = true;
const QString parPostMsgValue = i18n("Resuming paragraph.");

const bool parPostSndCheckValue = false;
const QString parPostSndValue = "";


// Make this a plug in.
typedef KGenericFactory<KCMKTTSD, QWidget> KCMKTTSDFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_kttsd, KCMKTTSDFactory("kcmkttsd") );

/**
 * Constructor
 * Makes the list of plug ins.
 * And the languages acording to the plug ins.
 */
KCMKTTSD::KCMKTTSD(QWidget *parent, const char *name, const QStringList &) : KCModule(KCMKTTSDFactory::instance(), parent, name){
    kdDebug() << "Running: KCMKTTSD::KCMKTTSD(QWidget *parent, const char *name, const QStringList &) : KCModule(KCMKTTSDFactory::instance(), parent, name)" << endl;

    //Defaults
    //textPreMsgValue = i18n("Paragraph interrumped. Message.");

    // Add the widget
    QVBoxLayout *layout = new QVBoxLayout(this);
    kttsdw = new KCMKTTSDWidget(this, "kttsdw");
    layout->addWidget(kttsdw);

    // Connect the signals from the KCMKTTSDWidget to this class
    connect( kttsdw, SIGNAL( addLanguage() ), this, SLOT( addLanguage() ) );
    connect( kttsdw, SIGNAL( configChanged() ), this, SLOT( configChanged() ) );
    connect( kttsdw, SIGNAL( removeLanguage() ), this, SLOT( removeLanguage() ) );
    connect( kttsdw, SIGNAL( setDefaultLanguage() ), this, SLOT( setDefaultLanguage() ) );
    connect( kttsdw, SIGNAL( updateRemoveButton() ), this, SLOT( updateRemoveButton() ) );
    connect( kttsdw, SIGNAL( updateDefaultButton() ), this, SLOT( updateDefaultButton() ) );


    // List of languages to be added, true add, false, leave.
    QMap<QString, bool> languagesToBeAdded;

    // Initializate the list of codes
    languagesMap["other"] = i18n("Other");
    for( QMap<QString, QString>::Iterator it = languagesMap.begin(); it != languagesMap.end(); ++it ){
        reverseLanguagesMap[it.data()] = it.key();
        languagesToBeAdded[it.key()] = false;
    }
    languagesToBeAdded["other"] = true;

    // Object for the KCMKTTSD configuration
    config = new KConfig("kttsdrc");

    // Set autoDelete to the list of structures of languages/plugins
    loadedLanguages.setAutoDelete(true);

    // Query for all the KCMKTTSD SynthPlugins and store the list in offers
    offers = KTrader::self()->query("KTTSD/SynthPlugin");

    // Iterate thru the posible plug ins and have them added to the combo box plugInSelection and make the list of posible languages
    for(unsigned int i=0; i < offers.count() ; ++i){
        kttsdw->plugInSelection->insertItem(offers[i]->name(), i);

        // Add the plug in to the combo box
        QStringList languages = offers[i]->property("X-KDE-Languages").toStringList();

        for( QStringList::Iterator it = languages.begin(); it != languages.end(); ++it ) {
            languagesToBeAdded[*it] = true;
            initLanguageCode (*it);
        }
    }

    // Insert the list of languages into the comobo box for language selection
    for( QMap<QString, bool>::Iterator it = languagesToBeAdded.begin(); it != languagesToBeAdded.end(); ++it ) {
        if(it.data()){
            kttsdw->languageSelection->insertItem(languagesMap[it.key()]);
        }
    }

    // Load the configuration from the file to the widgets :P
    load();
} 
  
/**
 * Destructor
 */
KCMKTTSD::~KCMKTTSD(){
   kdDebug() << "Running: KCMKTTSD::~KCMKTTSD()" << endl;
}
  
/**
 * This method is invoked whenever the module should read its 
 * configuration (most of the times from a config file) and update the 
 * user interface. This happens when the user clicks the "Reset" button in
 * the control center, to undo all of his changes and restore the currently
 * valid settings. NOTE that this is not called after the modules is loaded,
 * so you probably want to call this method in the constructor.
 */
void KCMKTTSD::load() {
    kdDebug() << "Running: KCMKTTSD::load()"<< endl;

    // Get rid of everything first for the sake of the reset button
    // Maybe we should do something here to jump to the right tab if the reset button was presed being in tab, anyway, that's not so bad
    QStringList languagesToRemove;
    for( QDictIterator<languageRelatedObjects> it( loadedLanguages ) ; it.current(); ++it ){
        languagesToRemove << it.currentKey();
    }
    for( QStringList::Iterator it = languagesToRemove.begin(); it != languagesToRemove.end(); ++it ) {
        removeLanguage(*it);
    }

    // Set the group general for the configuration of kttsd itself (no plug ins)
    config->setGroup("General");

    // Load the configuration of the text interruption messages and sound
    kttsdw->textPreMsgCheck->setChecked(config->readBoolEntry("TextPreMsgEnabled", textPreMsgCheckValue));
    kttsdw->textPreMsg->setText(config->readEntry("TextPreMsg", textPreMsgValue));
    kttsdw->textPreMsg->setEnabled(kttsdw->textPreMsgCheck->isChecked());

    kttsdw->textPreSndCheck->setChecked(config->readBoolEntry("TextPreSndEnabled", textPreSndCheckValue));
    kttsdw->textPreSnd->setURL(config->readPathEntry("TextPreSnd", textPreSndValue));
    kttsdw->textPreSnd->setEnabled(kttsdw->textPreSndCheck->isChecked());

    kttsdw->textPostMsgCheck->setChecked(config->readBoolEntry("TextPostMsgEnabled", textPostMsgCheckValue));
    kttsdw->textPostMsg->setText(config->readEntry("TextPostMsg", textPostMsgValue));
    kttsdw->textPostMsg->setEnabled(kttsdw->textPostMsgCheck->isChecked());

    kttsdw->textPostSndCheck->setChecked(config->readBoolEntry("TextPostSndEnabled", textPostSndCheckValue));
    kttsdw->textPostSnd->setURL(config->readPathEntry("TextPostSnd", textPostSndValue));
    kttsdw->textPostSnd->setEnabled(kttsdw->textPostSndCheck->isChecked());

    // Load the configuration of the par interruption messages and sound
    kttsdw->parPreMsgCheck->setChecked(config->readBoolEntry("ParPreMsgEnabled", parPreMsgCheckValue));
    kttsdw->parPreMsg->setText(config->readEntry("ParPreMsg", parPreMsgValue));
    kttsdw->parPreMsg->setEnabled(kttsdw->parPreMsgCheck->isChecked());

    kttsdw->parPreSndCheck->setChecked(config->readBoolEntry("ParPreSndEnabled", parPreSndCheckValue));
    kttsdw->parPreSnd->setURL(config->readPathEntry("ParPreSnd", parPreSndValue));
    kttsdw->parPreSnd->setEnabled(kttsdw->parPreSndCheck->isChecked());

    kttsdw->parPostMsgCheck->setChecked(config->readBoolEntry("ParPostMsgEnabled", parPostMsgCheckValue));
    kttsdw->parPostMsg->setText(config->readEntry("ParPostMsg", parPostMsgValue));
    kttsdw->parPostMsg->setEnabled(kttsdw->parPostMsgCheck->isChecked());

    kttsdw->parPostSndCheck->setChecked(config->readBoolEntry("ParPostSndEnabled", parPostSndCheckValue));
    kttsdw->parPostSnd->setURL(config->readPathEntry("ParPostSnd", parPostSndValue));
    kttsdw->parPostSnd->setEnabled(kttsdw->parPostSndCheck->isChecked());

    QString defaultLanguage = config->readEntry("DefaultLanguage");

    // Iterate thru loaded languages and load them and their configuration
    QStringList langs = config->groupList().grep("Lang_");
    for( QStringList::Iterator it = langs.begin(); it != langs.end(); ++it ) {
        QString langcode = (*it).right((*it).length()-5);
        kdDebug() << "Loading: " << *it << " Langcode: " << langcode << endl;
        config->setGroup(*it);
        addLanguage( langcode, config->readEntry("PlugIn"));
        if(!loadedLanguages.isEmpty()){
            loadedLanguages[langcode]->plugIn->load(config, *it);
        }
    }

    // we need the languages loaded to set the default one, chicken and egg, never liked chicken
    setDefaultLanguage(defaultLanguage);
    updateDefaultButton();
    updateRemoveButton();
} 
  
/**
 * This function gets called when the user wants to save the settings in 
 * the user interface, updating the config files or wherever the 
 * configuration is stored. The method is called when the user clicks "Apply" 
 * or "Ok".
 */
void KCMKTTSD::save() {
   kdDebug() << "Running: KCMKTTSD::save()"<< endl;
   // Clean up config before saving everything (as kconfig merges, when a language is removed from memory we have to ensure that is removed from disk too)
   QStringList allGroups = config->groupList();
   for( QStringList::Iterator it = allGroups.begin(); it != allGroups.end(); ++it ) {
      config->deleteGroup(*it);
   }

   // Set the group general for the configuration of kttsd itself (no plug ins)
   config->setGroup("General");

   // Set text interrumption messages and paths
   config->writeEntry("TextPreMsgEnabled", kttsdw->textPreMsgCheck->isChecked());
   config->writeEntry("TextPreMsg", kttsdw->textPreMsg->text());

   config->writeEntry("TextPreSndEnabled", kttsdw->textPreSndCheck->isChecked()); 
   config->writePathEntry("TextPreSnd", kttsdw->textPreSnd->url());

   config->writeEntry("TextPostMsgEnabled", kttsdw->textPostMsgCheck->isChecked());
   config->writeEntry("TextPostMsg", kttsdw->textPostMsg->text());

   config->writeEntry("TextPostSndEnabled", kttsdw->textPostSndCheck->isChecked());
   config->writePathEntry("TextPostSnd", kttsdw->textPostSnd->url());

   // Set par interrumption messages and paths
   config->writeEntry("ParPreMsgEnabled", kttsdw->parPreMsgCheck->isChecked());
   config->writeEntry("ParPreMsg", kttsdw->parPreMsg->text());

   config->writeEntry("ParPreSndEnabled", kttsdw->parPreSndCheck->isChecked()); 
   config->writePathEntry("ParPreSnd", kttsdw->parPreSnd->url());

   config->writeEntry("ParPostMsgEnabled", kttsdw->parPostMsgCheck->isChecked());
   config->writeEntry("ParPostMsg", kttsdw->parPostMsg->text());

   config->writeEntry("ParPostSndEnabled", kttsdw->parPostSndCheck->isChecked());
   config->writePathEntry("ParPostSnd", kttsdw->parPostSnd->url());

   // Iterate thru loaded languages and store their configuration
   for(QDictIterator<languageRelatedObjects> it(loadedLanguages); it.current() ; ++it){
      kdDebug() << "Saving: " << it.currentKey() << endl;
      
      // Let's check if this is the default language
      if(it.current()->listItem->pixmap(2)){
         // If it is, store it, and store it in General
         config->setGroup("General");
         config->writeEntry("DefaultLanguage", it.currentKey());
      }
      config->setGroup(QString("Lang_")+it.currentKey());
      config->writeEntry("PlugIn", it.current()->plugInName);
      it.current()->plugIn->save(config, QString("Lang_")+it.currentKey());
   }

   config->sync();

/*
   // To be reviewed, to use KSpeech when ready
   if(KMessageBox::questionYesNo(this, i18n("Do you want to reload the Text To Speech configuration ?"), i18n("Reload configuration ?")) == KMessageBox::Yes){
      kdDebug() << "Restarting KTTSD" << endl;
      DCOPClient *client = kapp->dcopClient();
      if(client->attach()){
         if(client->send("kttsd", "kspeech", "void exit()", "")){
            kdDebug() << "Killing KTTSD worked!" << endl;
         } else {
            kdDebug() << "KTTSD not running or some other kind of error" << endl;
         }
      } else {
         kdDebug() << "Couldn't attach" << endl;
      }
      kdDebug() << "Starting KTTSD" << endl;
      KProcess *proc = new KProcess;
      *proc << "kttsd";
      proc->start(KProcess::DontCare);      
   } else {
      kdDebug() << "NOT restarting KTTSD by user option" << endl;
   }*/
} 
  
/**
 * This function is called to set the settings in the module to sensible
 * default values. It gets called when hitting the "Default" button. The 
 * default values should probably be the same as the ones the application 
 * uses when started without a config file.
 */
void KCMKTTSD::defaults() {
   kdDebug() << "Running: KCMKTTSD::defaults()"<< endl;

   kttsdw->textPreMsgCheck->setChecked(textPreMsgCheckValue);
   kttsdw->textPreMsg->setText(textPreMsgValue);

   kttsdw->textPreSndCheck->setChecked(textPreSndCheckValue);
   kttsdw->textPreSnd->setURL(textPreSndValue);

   kttsdw->textPostMsgCheck->setChecked(textPostMsgCheckValue);
   kttsdw->textPostMsg->setText(textPostMsgValue);

   kttsdw->textPostSndCheck->setChecked(textPostSndCheckValue);
   kttsdw->textPostSnd->setURL(textPostSndValue);

   kttsdw->parPreMsgCheck->setChecked(parPreMsgCheckValue);
   kttsdw->parPreMsg->setText(parPreMsgValue);

   kttsdw->parPreSndCheck->setChecked(parPreSndCheckValue);
   kttsdw->parPreSnd->setURL(parPreSndValue);

   kttsdw->parPostMsgCheck->setChecked(parPostMsgCheckValue);
   kttsdw->parPostMsg->setText(parPostMsgValue);

   kttsdw->parPostSndCheck->setChecked(parPostSndCheckValue);
   kttsdw->parPostSnd->setURL(parPostSndValue);
}

/**
 * This is a static method which gets called to realize the modules settings
 * durign the startup of KDE. NOTE that most modules do not implement this
 * method, but modules like the keyboard and mouse modules, which directly
 * interact with the X-server, need this method. As this method is static,
 * it can avoid to create an instance of the user interface, which is often
 * not needed in this case.
 */
void KCMKTTSD::init(){
   kdDebug() << "Running: KCMKTTSD::init()"<< endl;
} 
  
/**
 * The control center calls this function to decide which buttons should
 * be displayed. For example, it does not make sense to display an "Apply" 
 * button for one of the information modules. The value returned can be set by 
 * modules using setButtons.
 */
int KCMKTTSD::buttons() {
   kdDebug() << "Running: KCMKTTSD::buttons()"<< endl;
   return KCModule::Ok|KCModule::Apply|KCModule::Help|KCModule::Reset;
} 

/**
 * This function returns the small quickhelp.
 * That is displayed in the sidebar in the KControl
 */
QString KCMKTTSD::quickHelp() const{
   kdDebug() << "Running: KCMKTTSD::quickHelp()"<< endl;
   return i18n("<h1>Text To Speech</h1>"
               "<p>This is the configuration for the text to speech dcop service</p>"
               "<p>This allows other applications to access text to speech resources</p>"
               "<p>Be sure to configure a default language for the language you are using as this will be the language used by most of the applications</p>");
}

const KAboutData* KCMKTTSD::aboutData() const{
   KAboutData *about =
   new KAboutData(I18N_NOOP("kttsd"), I18N_NOOP("KCMKTTSD"),
                  0, 0, KAboutData::License_GPL,
                  I18N_NOOP("(c) 2002, Jos�Pablo Ezequiel Fern�dez"));

   about->addAuthor("Jos�Pablo Ezequiel Fern�dez", I18N_NOOP("Author") , "pupeno@kde.org");
   return about;
}
 
/**
 * Loads the configuration plug in for a specific plug in
 */
PlugInConf *KCMKTTSD::loadPlugIn(const QString &plugInName){
   kdDebug() << "Running: KCMKTTSD::loadPlugIn(const QString &plugInName)"<< endl;

   // Iterate thru the plug in offers to find the plug in that matches the plugInName
   for(unsigned int i=0; i < offers.count() ; ++i){
      // Compare the plug in to be loeaded with the entry in offers[i]
      kdDebug() << "Comparing " << offers[i]->name() << " to " << plugInName << endl;
      if(offers[i]->name() == plugInName){
      // When the entry is found, load the plug in
      // First create a factory for the library
      KLibFactory *factory = KLibLoader::self()->factory(offers[i]->library());
      if(factory){
         // If the factory is created succesfully, isntatiate the PlugInConf class for the specific plug in to get the plug in configuration object.
         PlugInConf *plugIn = KParts::ComponentFactory::createInstanceFromLibrary<PlugInConf>(offers[i]->library(), NULL, offers[i]->library());
         if(plugIn){
            // If everything went ok, return the plug in pointer
            return plugIn;
         } else {
            // Something went wront, returning null
            return NULL;
         }
      } else {
         // Something went wront, returning null
         return NULL;
      }
      break;
      }
   }
   // The plug in was not found (unexpected behaviour, returns null)
   return NULL;
}
    
/**
 * Add a language
 * This is a wrapper function that takes the parameters for the real addLanguage from the
 * widgets to later call it.
 */
void KCMKTTSD::addLanguage(){
   kdDebug() << "Running: KCMKTTSD::addLanguage()"<< endl;
   kdDebug() << "Adding plug in " << kttsdw->plugInSelection->currentText() << " for language " << kttsdw->languageSelection->currentText() << endl;

   if(loadedLanguages.find(reverseLanguagesMap[kttsdw->languageSelection->currentText()]) == 0){
      if(reverseLanguagesMap[kttsdw->languageSelection->currentText()] == "other"){
         QString customLanguage = KLineEditDlg::getText(i18n("Create custom language"), i18n("Please enter the code for the custom language:"));
         addLanguage( customLanguage, kttsdw-> plugInSelection->currentText());
      } else {
         addLanguage( reverseLanguagesMap[kttsdw->languageSelection->currentText()], kttsdw->plugInSelection->currentText());
      }
   } else {
      KMessageBox::error(0, i18n("This language already has a plug in assigned. Please remove it before re-assigning it"), i18n("Language not valied"));
   }
}

/**
 * Add a language to the pool of languages for the lang/plugIn pair
 */
void KCMKTTSD::addLanguage(const QString &language, const QString &plugInName){
   kdDebug() << "Running: KCMKTTSD::addLanguage(const QString &lang, const QString &plugInName)"<< endl;
   kdDebug() << "Adding plug in " << plugInName << " for language " << language << endl;

   // If the language is not in the map, let's add it.
   if(!languagesMap.contains(language)){
      initLanguageCode (language);
   }

   // This object will contain pointers to all the objects related to a single language/plug in pair.
   kdDebug() << "Creating newLanguage" << endl;
   languageRelatedObjects *newLanguage = new languageRelatedObjects();

   // Load the plug in
   kdDebug() << "Creating the new plugIn" << endl;
   newLanguage->plugIn = loadPlugIn(plugInName);

   // If there wasn't any error, let's go on
   if(newLanguage->plugIn){
      // Create the item in the list and add it and store the pointer in the structure
      kdDebug() << "Creating the new listItem" << endl;

      newLanguage->listItem = new KListViewItem(kttsdw->languagesList, languagesMap[language], plugInName);

      // Store the name of the plug in in the structure
      kdDebug() << "Storing the plug in name" << endl;
      newLanguage->plugInName = plugInName;

      // Add a tab with the new plug in configuration widget.
      kdDebug() << "Adding the tab" << endl;
      kttsdw->mainTab->addTab( newLanguage->plugIn, languagesMap[language]);

      // Let plug in changes be as global changed to show apply button.
      kdDebug() << "Connecting" << endl;
      connect(  newLanguage->plugIn, SIGNAL( changed(bool) ), this, SLOT( configChanged() )  );

      // Let's insert it in the QDict to keep track of language structure
      kdDebug() << "Inserting the newLanguage in the QDict" << endl;
      loadedLanguages.insert( language, newLanguage);

      kdDebug() << "Done" <<endl;
   } else {
      KMessageBox::error(0, i18n("Speech syntheziser plug in library not found or corrupted"), i18n("Plug in not found"));
   }
}

/**
  * Initializes the language with the given language code by determining
  * its name.
 */
void KCMKTTSD::initLanguageCode (const QString &code) {
   if(!languagesMap.contains(code)) {
      QString name = QString::null;

      QString file = locate("locale", QString::fromLatin1("%1/entry.desktop").arg(code));
      if (!file.isNull() && !file.isEmpty()) {
         KSimpleConfig entry(file);
         entry.setGroup("KCM Locale");
         name = entry.readEntry("Name", QString::null);
      }

      if (name.isNull() || name.isEmpty())
         name = code;
      else
         name = name + QString::fromLatin1(" (%1)").arg(code);

      languagesMap[code] = name;
      reverseLanguagesMap[name] = code;
   }
}

/**
 * Remove language 
 * This is a wrapper function that takes the parameters for the real removeLanguage from the
 * widgets to later call it.
 */
void KCMKTTSD::removeLanguage(){
   kdDebug() << "Running: KCMKTTSD::removeLanguage()"<< endl;

   // Get the selected language
   QListViewItem *itemToRemove = kttsdw->languagesList->selectedItem();

   // Call the real removeLanguage function
   if(itemToRemove){
      removeLanguage(reverseLanguagesMap[itemToRemove->text(0)]);
   }
}

/**
 * Remove a language named lang
 */
void KCMKTTSD::removeLanguage(const QString &language){
   kdDebug() << "Running: KCMKTTSD::removeLanguage(const QString &language)"<< endl;
   if(loadedLanguages[language]){
      // Remove the KListViewItem object
      kttsdw->languagesList->takeItem(loadedLanguages[language]->listItem);

      // Remove the configuration widget
      delete loadedLanguages[language]->plugIn;

      // And finish removing the wholestructure from memory
      loadedLanguages.remove(language);
   }
}

/**
 * Set default langauge
 * This is a wrapper function that takes the parameters for the real setDefaultLanguage from the
 * widgets to later call it.
 */
void KCMKTTSD::setDefaultLanguage(){
   kdDebug() << "Running: KCMKTTSD::setDefaultLanguage()"<< endl;
   // Get the selected language
   QListViewItem *defaultLanguageItem = kttsdw->languagesList->selectedItem();

   // Call the real setDefaultLanguage function
   if(defaultLanguageItem){
      setDefaultLanguage(reverseLanguagesMap[defaultLanguageItem->text(0)]);
   }
}

/**
 * Set the default language
 */
void KCMKTTSD::setDefaultLanguage(const QString &defaultLanguage){
   kdDebug() << "Running: KCMKTTSD::setDefaultLanguage(const QString &defaultLanguage)"<< endl;
   // Yes should be a beautiful tick icon
   for(QDictIterator<languageRelatedObjects> it(loadedLanguages); it.current() ; ++it){
      if(it.currentKey() == defaultLanguage){
         kdDebug() << "Setting " << it.currentKey() << " as the default language" << endl;
         it.current()->listItem->setPixmap(2, DesktopIcon("ok", 16));
      } else {
         kdDebug() << "UN-Setting " << it.currentKey() << " as the default language" << endl;
         it.current()->listItem->setPixmap(2, NULL);
      }
   }
   updateDefaultButton();
}

/**
 * Update the status of the Remove button
 */
void KCMKTTSD::updateRemoveButton(){
   kdDebug() << "Running: KCMKTTSD::updateRemoveButton()"<< endl;
   if(kttsdw->languagesList->selectedItem()){
      kttsdw->removeLanguageButton->setEnabled(true);
   } else {
      kttsdw->removeLanguageButton->setEnabled(false);
   }
}

/**
 * Update the status of the Default button
 */
void KCMKTTSD::updateDefaultButton(){
   kdDebug() << "Running: KCMKTTSD::updateDefaultButton()"<< endl;
   if(kttsdw->languagesList->selectedItem()){
      if(!kttsdw->languagesList->selectedItem()->pixmap(2)){
         kttsdw->makeDefaultLanguage->setEnabled(true);
      } else {
         kttsdw->makeDefaultLanguage->setEnabled(false);
      }
   } else {
      kttsdw->makeDefaultLanguage->setEnabled(false);
   }
}

